/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.docware;

import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.PrintWriter;

import casa.dodwan.util.Bufferizer;
import casa.dodwan.util.BufferAccessException;
import casa.dodwan.util.Logger;

public class DescriptorBufferizer extends Bufferizer<Descriptor> {

    // ------------------------------------------------------------
    public Descriptor fromStream(InputStream is) throws Exception {
	
	Descriptor result = new Descriptor();

	BufferedReader reader = new BufferedReader(new InputStreamReader(is));
	while (reader.ready()) {
	    String line = reader.readLine();
	    if (! line.startsWith("#")) {
		int idx = line.indexOf('=');
		String name = line.substring(0, idx);
		String value = line.substring(idx+1);
		result.setAttribute(name, value);
	    }
	}
	return result;
    }
 
    // ------------------------------------------------------------
    public void toStream(Descriptor desc, OutputStream out) 
	throws Exception {

	PrintWriter writer = new PrintWriter(out);
	for (String name: desc.getNames()) {
	    String value = desc.getAttribute(name);
	    writer.println(name + "=" + value);
	}
	writer.flush();
    }


    // ------------------------------------------------------------
    public Descriptor fromBuffer(byte[] buffer, int offset, int length)
	throws BufferAccessException {

	// Logger.log("dbuf", "fromBuffer(" + buffer
	// 	   + ", " + offset
	// 	   + ", " + length
	// 	   + ")");

	Descriptor result = null;
	try {
	    InputStream is = new ByteArrayInputStream(buffer, offset, length);
	    result = fromStream(is);
	} 
	catch (Exception e) {
	    Logger.log("dbuf", "Failed to decode descriptor");
	    // e.printStackTrace();
	    throw new BufferAccessException();
	}

	return result;
    }

    // ------------------------------------------------------------
    public int toBuffer(Descriptor desc, byte[] buffer, int offset)
	throws BufferAccessException {

	// Logger.log("dbuf", "toBuffer(desc, " + buffer
	//  	   + ", " + offset
	//  	   + ")");

	byte[] buf = toBuffer(desc);
	int length = buf.length;
	System.arraycopy(buf, 0, buffer, offset, length);

	return length;
    }

    // ------------------------------------------------------------
    public byte[] toBuffer(Descriptor desc)
	throws BufferAccessException {

	// Logger.log("dbuf", "toBuffer(desc)");

	byte[] result = null;

	try {
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    toStream(desc, baos);
	    result = baos.toByteArray();
	}
	catch (Exception e) {
	    throw new BufferAccessException();
	}

	return result;
    }

    // ============================================================
    /**
     * @undocumented
     */
    public static void main(String[] args) throws Exception {

	Descriptor d1 = new Descriptor();
	d1.setAttribute("att1", "fg");
	d1.setAttribute("att2", "rg jg");
	d1.setAttribute("att3", "pg");

	System.out.println("d1=" + d1);

	Bufferizer<Descriptor> db = new DescriptorBufferizer();
	db.toFile(d1, "/tmp/d1");

	Descriptor tmp = db.fromFile("/tmp/d1");
	System.out.println("d1 from file ok: " + tmp.equals(d1));
    }

}
