/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.docware;

import casa.dodwan.util.Selector;
import casa.dodwan.util.Verbosity;

public class DescriptorSelector 
    implements Selector<Descriptor> {

	private Descriptor regex_;

	public Verbosity verbosity = new Verbosity();

	// ------------------------------------------------------------
	public DescriptorSelector(Descriptor regex) {
		regex_ = regex;
	}

	// ------------------------------------------------------------
	public Descriptor getRegex() {
		return regex_;
	}

	// ------------------------------------------------------------
	public boolean isMatchedBy(Descriptor value) {

		boolean result;
		try {
		    result = value.matches(regex_);
		} catch (Exception e) {
		    result = false;
		}
		
		if (verbosity.isEnabled())
			System.out.println("DescriptorSelector[" + regex_ + "].isMatchedBy("
					+ value + "): " + result);

		return result;
	}

	// ------------------------------------------------------------
	public void setRegex(Descriptor regex) {
		regex_ = regex;
	}
}
