/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.docware;

import casa.dodwan.util.BufferAccessException;
import casa.dodwan.util.Bufferizer;
import casa.dodwan.util.DataBufferizer;

public class ProfileBufferizer extends Bufferizer<Profile> {

    private Bufferizer<Descriptor> descriptorBufferizer_
	= new DescriptorBufferizer();

    // ------------------------------------------------------------
    public Profile fromBuffer(byte[] buffer, int offset, int length)
	throws BufferAccessException {

	// System.out.println("ProfileBufferizer.fromBuffer("
	// 		   + buffer + ", "
	// 		   + offset + ", "
	// 		   + length + ")");

	Profile result = new Profile();
	int rlength = 0;
	try {
	    while(rlength < length) {
		// Reading descriptor
		int dLength = DataBufferizer.readInt(buffer, offset);
		Descriptor desc = 
		    descriptorBufferizer_.fromBuffer(buffer, offset + 4, dLength);
		// System.out.println(">Desc=" + desc);
		offset += dLength + 4;
		rlength += dLength + 4;
		result.add(desc);
	    }
	}
	catch (Exception e) {
	    throw new BufferAccessException();
	}

	return result;
    }

    // ------------------------------------------------------------
    public byte[] toBuffer(Profile prof) {

	// There's no way to guess the size of the buffer that can
	// accommodate this profile, so we must try successive sizes
	// until one fits

	int size = 8192;
	int length = -1;
	byte[] buf = null;
	while (length == -1) {
	    try {
		size = 2 * size;
		buf = new byte[size];
		length = toBuffer(prof, buf, 0);
	    }
	    catch (Exception e) {
		e.printStackTrace();}
	}

	byte[] result = new byte[length];
	System.arraycopy(buf, 0, result, 0, length);

	return result;
    }

    // ------------------------------------------------------------
    public int toBuffer(Profile prof, byte[] buffer, int offset)
	throws BufferAccessException {

	int length = 0;
	
	try {
	    for (Descriptor desc: prof.descriptors()) {
		//System.out.println("<Desc=" + desc);
		int dLength = descriptorBufferizer_.toBuffer(desc, buffer, offset + 4);
		DataBufferizer.writeInt(dLength, buffer, offset);
		
		offset += dLength + 4;
		length += dLength + 4;
	    }
	}
	catch (Exception e) {
	    throw new BufferAccessException();
	}
	
	// System.out.println("ProfileBufferizer.toBuffer(prof, "
	// 		   + buffer + ", "
	// 		   + offset + ") -> "
	// 		   + length);
	
	return length;
    }
    
    // ============================================================
    /**
     * @undocumented
     */
    public static void main(String[] args) throws Exception {

	Descriptor d1 = new Descriptor();
	d1.setAttribute("att1", "fg");
	d1.setAttribute("att2", "rg jg");
	d1.setAttribute("att3", "pg");
	Descriptor d2 = new Descriptor();
	d2.setAttribute("att1", "f.*");
	d2.setAttribute("att2", ".*rg.*");

	Profile prof = new Profile();
	prof.add(d1);
	prof.add(d2);

	System.out.println("prof=" + prof);

	Bufferizer<Profile> cb = new ProfileZipBufferizer();
	cb.toFile(prof, "/tmp/prof");

	Profile tmp = cb.fromFile("/tmp/prof");
	System.out.println("prof from file=" + tmp);
 	
    }

}
