/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.hub;

import java.io.BufferedReader;
import java.io.PrintStream;

import casa.dodwan.util.Console;
import casa.dodwan.util.ArgumentNotFoundException;
import casa.dodwan.util.ArgumentParsing;
import casa.dodwan.util.ArgumentParsingException;
import casa.dodwan.util.Command;
import casa.dodwan.util.Domain;

public class BridgeConsole extends Console {
    
 
    // ============================================================
    // MAIN COMMANDS
    // ============================================================
    
    public Bridge bridge_;
    
    // ------------------------------------------------------------
    public BridgeConsole(Bridge bridge) {
	
	bridge_ = bridge;
	prompt_ = "% ";
	
	addSubConsole("ca", "[ca]che", bridge_.cacheService.console());
    }
    
    // ============================================================
    // COMMAND PARSING
    // ============================================================
    
    // ------------------------------------------------------------
    @Override
	public void help(String offset, PrintStream out) {
	out
	    .println(offset + "[a]dd <domainId>\n"
		     + offset + "[rem]ove <domainId>\n"
		     + "[su]spend / [res]ume\n"
		     + offset
		     + "[on]line / [off]line\n"
		     + offset
		     + "[st]atus\n"
		     + offset
		     + "[quit]\n");

	subHelp(offset, out);  
    }

    // ------------------------------------------------------------
    @Override
	public void processCommand(String[] arguments, BufferedReader in,
				   PrintStream out) {
	
	// Process commands locally
	if (arguments[0].startsWith("a")) {
	    addDomain(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("rem")) {
	    removeDomain(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("on")) {
	    online(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("off")) {
	    offline(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("su")) {
	    suspend(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("res")) {
	    resume(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("st")) {
	    status(arguments, out);
	    return;
	}

	if (arguments[0].startsWith("quit")) {
	    quit(arguments, out);
	    return;
	}

  	super.processCommand(arguments, in, out);

    }
    
    // ------------------------------------------------------------
    public void addDomain(String[] arguments, PrintStream out) {
	
	if (arguments.length != 2) {
	    help("", out);
	    return;
	}

	String id = arguments[1];
	Domain dom = bridge_.getDomain(id);
	if (dom != null) {
	    out.println("Domain " + id + " already exists");
	    return;
	}
	
	bridge_.addDomain(id);
	out.println("Domain " + id + " added");
    }

    // ------------------------------------------------------------
    public void removeDomain(String[] arguments, PrintStream out) {
	
	if (arguments.length != 2) {
	    help("", out);
	    return;
	}

	String id = arguments[1];
	Domain dom = bridge_.getDomain(id);
	if (dom == null) {
	    out.println("Domain " + id + " does not exist");
	    return;
	}
	
	bridge_.removeDomain(id);
	out.println("Domain " + id + " removed");
    }

    // ------------------------------------------------------------
    public void resume(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}
	
	bridge_.resume();
    }
    
    // ------------------------------------------------------------
    public void suspend(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	bridge_.suspend();
    }

    // ------------------------------------------------------------
    public void online(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}
	
	bridge_.online();
    }
    
    // ------------------------------------------------------------
    public void offline(String[] arguments, PrintStream out) {
	
	bridge_.offline();
    }

    // ------------------------------------------------------------
    public void status(String[] arguments, PrintStream out) {
	
	out.println("Bridge is "
		    + (bridge_.isOnline()?"online":"offline"));
    }

    // ------------------------------------------------------------
    public void resetLogger(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	bridge_.resetLogger();
    }

    // ------------------------------------------------------------
    public void quit(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	bridge_.suspend();
	bridge_.doNotify();
    }
	
}
