/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.hub;

import java.io.BufferedReader;
import java.io.PrintStream;

import casa.dodwan.util.Console;
import casa.dodwan.util.ArgumentNotFoundException;
import casa.dodwan.util.ArgumentParsing;
import casa.dodwan.util.ArgumentParsingException;
import casa.dodwan.util.Command;
import casa.dodwan.util.Environment;

public class HubConsole extends Console {
    
 
    // ============================================================
    // MAIN COMMANDS
    // ============================================================
    
    public Hub hub_;
    
    // ------------------------------------------------------------
    public HubConsole(Hub hub) {
	
	hub_ = hub;
	prompt_ = "% ";
	
	// addSubConsole("t", "[t]ransmission", hub_.transmissionService.console());
    }
    
    // ============================================================
    // COMMAND PARSING
    // ============================================================
    
    // ------------------------------------------------------------
    @Override
	public void help(String offset, PrintStream out) {
	out
	    .println(offset
		     + "[su]spend / [resu]me\n"
		     + offset
		     + "[on]line / [off]line\n"
		     + offset
		     + "[req]uestLog <-h host> <-p pattern> <-l hh:mm:ss>\n"
		     + offset
		     + "[rese]tLogger\n"
		     + offset
		     + "[st]atus\n"
		     + offset
		     + "[quit]\n");

	subHelp(offset, out);  
    }



    // ------------------------------------------------------------
    @Override
	public void processCommand(String[] arguments, BufferedReader in,
				   PrintStream out) {
	
	// Process commands locally
	if (arguments[0].startsWith("req")) {
	    requestLog(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("rese")) {
	    resetLogger(arguments, out);
	    return;
	}

	if (arguments[0].startsWith("quit")) {
	    quit(arguments, out);
	    return;
	}

  	super.processCommand(arguments, in, out);

    }
 
    // ------------------------------------------------------------
    public void requestLog(String[] arguments, PrintStream out) {

	String host = arguments[1];
	String pattern = arguments[2];

        long lifetime = 300000;
        
        try {
            host = ArgumentParsing.searchString("-h", arguments);
            pattern = ArgumentParsing.searchString("-p", arguments);
            
            try {
                lifetime = ArgumentParsing.search_hh_mm_ss("-l", arguments);
            }
            catch (ArgumentNotFoundException e) {}
	}
	catch (ArgumentNotFoundException e) {
	    Command.missingArgument(e);
	    return;
	} 
	catch (ArgumentParsingException e) {
	    Command.parsingException(e);
	    return;
	}
	catch (Exception e) {
	    out.println(e);
	}

    }
    
    // ------------------------------------------------------------
    public void resetLogger(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	hub_.resetLogger();
    }

    // ------------------------------------------------------------
    public void quit(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	// hub_.doNotify();
    }
	
}
