/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.net;

import java.io.InputStream;
import java.io.OutputStream;

import casa.dodwan.util.DataBufferizer;
import casa.dodwan.util.Logger;

public class Stream_IO {
 
    // ------------------------------------------------------------
    private static byte chksum(byte[] data,
			       int offset, int length) {
	byte result = 0;
	for (int i=0; i<length; i++)
	    result += data[i+offset];
	return result;
    }

    // ------------------------------------------------------------
    public static byte[] receiveBytes(InputStream in, int length) 
	throws Exception {
	
	byte[] result = new byte[length];
	
	int offset = 0;
	int expected = length;
	while (expected > 0) {
	    int nr = in.read(result, offset, expected);
	    if (nr <= 0)
		throw new Exception();
	    expected -= nr;
	    offset += nr;
	}
	
	// Logger.log("stream_io", "< [" + result.length + "]");
	
	return result;
    }

    // ------------------------------------------------------------
    public static byte[] receiveBundle(InputStream  in) 
	throws Exception {
	
	// Let us first read a 4-byte header, and thus learn how many
	// data bytes must be consumed after this header
	byte[] header = receiveBytes(in, 4);
	int length = DataBufferizer.readInt(header, 0);

	byte[] result = receiveBytes(in, length);
	
	// Logger.log("stream_io", "< bundle[" + result.length + "]");
	// Logger.log("stream_io", "< bundle[" + result.length 
	// 	   + "],chksum=" + chksum(result, 0, length));

	return result;
    }

    // ------------------------------------------------------------
    public static void sendBytes(OutputStream out, byte[] data,
				 int offset, int length) 
	throws Exception {

	out.write(data, offset, length);
	out.flush();

	// Logger.log("stream_io", "> [" + length + "]");
    }

    // ------------------------------------------------------------
    public static void sendBytes(OutputStream out, byte[] data) 
	throws Exception {

	sendBytes(out, data, 0, data.length);
    }

    // ------------------------------------------------------------
    public static void sendBundle(OutputStream out, 
				  byte[] data,
				  int offset, 
				  int length) 
	throws Exception {

	// Let us first send a 4-byte header, and thus tell the peer how many
	// data bytes it should read after this header
	byte[] header = new byte[4];
	DataBufferizer.writeInt(length, header, 0);
	out.write(header);

	sendBytes(out, data, offset, length);

	// Logger.log("stream_io", "> bundle[" + length + "]");
	// Logger.log("stream_io", "> bundle[" + length 
	// + ",chksum=" + chksum(data, offset, length));
    }

    // ------------------------------------------------------------
    public static void sendBundle(OutputStream out, 
				  byte[] data) 
	throws Exception {

	sendBundle(out, data, 0, data.length);
    }

}
