/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.pubsub;

import java.io.OutputStream;
import java.io.File;
import java.io.PrintStream;

import casa.dodwan.message.Message;
import casa.dodwan.docware.Descriptor;
import casa.dodwan.util.Processor;
import casa.dodwan.pubsub.PubSubService;

public class MessageSaver implements Processor<Descriptor> {

    private File dir_;
    private PubSubService pubsub_;

    public MessageSaver(File dir, PubSubService pubsub) {

	// System.out.println("Creating message saver for " + dir);
	dir_ = dir;
	pubsub_ = pubsub;
    }
    
    // ------------------------------------------------------------
    public void process(Descriptor desc) throws Exception {

	// Extracting attributes from descriptor
	String key = desc.getDocumentId();
	String src = desc.getAttribute("src");
	String fname = desc.getAttribute("file");

	// Retrieving payload from cache
	// byte[] payload = pubsub_.getAsBytes(key);

	// Processing message
	System.out.println("Saving msg " + key + " in " + dir_);

	String target = null;
	if (fname == null) {
	    target = src + "_" + key;
	    System.out.println("This message does not contain a file. Saving in " + target);
	}
	else {
	    target = src + "_" + fname;
	    System.out.println("This message contains a file named " + fname
			       + ", saving in " + target);
	}
	
	// Saving message payload in local file
	try {
	    File file = new File(dir_, target);
	    pubsub_.getAsFile(key, file);
	    // PrintStream out = new PrintStream(file);
	    // out.write(payload);
	    // out.flush();
	    // out.close();
	    // System.out.println("Message payload saved in " + target);
	}
	catch (Exception e) {
	    System.out.println("Failed to write " + target);
	}
    }
	

    // ------------------------------------------------------------
    public String toString() {
	
	return "MessageSaver(dir=" + dir_ + ")";
    }
}
