/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.pubsub;

import casa.dodwan.docware.Descriptor;
import casa.dodwan.util.Processor;



/**
   Defines the interface for an object that listens to events pertaining to the
   DoDWAN pub/sub service.  Two events are handled: the publication of a
   message and the reception of a new message, result of a preceding
   subscription.
*/
public interface PubSubListener {

    // ------------------------------------------------------------
    /**
       Invoked after a message has been published.
       @param desc the descriptor of the message that has been published.
    */
    public void publish(Descriptor desc);

    // ------------------------------------------------------------
    /**
       Invoked after a message has been received. 
       @param msg the message that has been received.
    *  @param proc the processor that has handled the received message.
    */
    public void newMessageReceived(Descriptor desc,
				   Processor<Descriptor> proc);

}
