/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;

public class Command {

	// ------------------------------------------------------------
	public static void exit(int error) {
		System.exit(error);
	}

	// ------------------------------------------------------------
	public static void missingArgument(ArgumentNotFoundException e) {
		System.out.println("\nArgument '" + e.argument + "' is missing...");
	}

	// ------------------------------------------------------------
	public static void parsingException(ArgumentParsingException e) {
		System.out.println("\nCould not parse argument for option '"
				+ e.argument + "'");
	}

	// ------------------------------------------------------------
	public static String readLine() {

		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(System.in));
		} catch (Exception e) {
			System.out.println("Command.readLine() failed");
			System.exit(1);
		}
		return readLine(reader);
	}

	// ------------------------------------------------------------
	public static String readLine(BufferedReader reader) {

		String result = null;
		try {
			result = reader.readLine();
		} catch (Exception e) {
			System.out.println("Command.readLine(BufferReader reader) failed");
			System.exit(1);
		}
		return result;
	}

	// ------------------------------------------------------------
	public static String readLine(String fname) {

		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(fname));
		} catch (Exception e) {
			System.out.println("Command.readLine(String fname) failed");
			System.exit(1);
		}
		return readLine(reader);
	}

}
