/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.DataFormatException;
import java.util.zip.Deflater;
import java.util.zip.Inflater;

/**
 * A <code>Compressor&lt;E&gt;</code> implements a strategy for storing an
 * object of type <code>E</code> in a buffer (see method <code>toBuffer()</code>
 * ), and for extracting an object of type <code>E</code> from a buffer (see
 * method <code>fromBuffer()</code>).
 * 
 * @author $Author: F. Guidec$
 * @version $Revision: 1.0$
 */

public class Compressor {
    
    // ------------------------------------------------------------
    /**
     * @undocumented
     */
    public static byte[] compress(byte[] src) 
	throws Exception {

	return compress(src, 0, src.length);
    }

    // ------------------------------------------------------------
    /**
     * @undocumented
     */
    public static byte[] compress(byte[] src, int offset, int length) 
	throws Exception {

	Deflater deflater = new Deflater(Deflater.FILTERED);
	deflater.setInput(src, offset, length);
	deflater.finish();
	
	ByteArrayOutputStream bos =
	    new ByteArrayOutputStream(src.length);
	
	byte[] buf = new byte[1024];
	while (!deflater.finished()) {
	    int count = deflater.deflate(buf);
	    bos.write(buf, 0, count);
	}
	try {
	    bos.close();
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
	
	return bos.toByteArray();
    }

    // ------------------------------------------------------------
    /**
     * @undocumented
     */
    public static byte[] uncompress(byte[] src, int offset, int length)
	throws Exception {
	    
	Inflater inflater = new Inflater();
	inflater.setInput(src, offset, length);

	ByteArrayOutputStream bos =
	    new ByteArrayOutputStream(src.length);

	byte[] buf = new byte[1024];
	while (!inflater.finished()) {
	    try {
		int count = inflater.inflate(buf);
		bos.write(buf, 0, count);
	    } catch (DataFormatException e) {
		e.printStackTrace();
	    }
	}

	try { bos.close(); 
	} catch (IOException e) { 
	    e.printStackTrace();
	}
	inflater.end();

	byte[] result = bos.toByteArray();

	return result;
    }
    
    // ------------------------------------------------------------
    /**
     * @undocumented
     */
    public static byte[] uncompress(byte[] src)
	throws Exception {
	    
	return uncompress(src, 0, src.length);
    }

    // ------------------------------------------------------------
    public static void main(String[] args) 
	throws Exception {

	java.util.Random random = new java.util.Random(System.currentTimeMillis());

	byte[] data = new byte[10*1000000];
	random.nextBytes(data);

	for (int cnt=1; cnt<10; cnt++) {
	    int size = cnt * 1000000;
	    
	    byte check1 = 0;
	    for (int i=0; i<size; i++) 
		check1 += data[i];
	    
	    long t1 = System.currentTimeMillis();
	    byte[] comp = compress(data, 0, size);
	    long t2 = System.currentTimeMillis();
	    float ratio = (float)size / (float)(comp.length);
	    int rate1 = size / (int)(t2 - t1); // / 1000;
	    
	    t1 = System.currentTimeMillis();
	    byte[] data2 = uncompress(comp);
	    t2 = System.currentTimeMillis();
	    int rate2 = size / (int)(t2 - t1); // / 1000;
	    
	    byte check2 = 0;
	    for (int i=0; i<data2.length; i++)
		check2 += data2[i];

	    if (check1 != check2)
		System.out.println("Invalid checksum!");

	    System.out.println("Cnt=" + cnt 
			       + "\tDeflate: " + rate1 + " kBps"
			       + "\tratio=" + ratio
			       + "\tInflate: " + rate2 + " kBps");
	}
    }
}


